/*
 * PID-controller
 * Upon construction, this class receives parameters for
 * a PID- controller. An error can then be sent to the
 * object, and a new velocity to reduce the error will be returned.
 * The two previous erros are stored in the object so that derivation
 * and integration of the error is possible (this is done in the formula).
 */

public class PID
{

	private double v;
	public double K;
	public double Ti;
	public double Ts;
	public double Td;
	public double v_1 = 0;
	public double error = 0;
	public double error_1 = 0;
	public double error_2 = 0;

	// Constructor
	public PID (double K, double Ts, double Ti, double Td)
	{
		this.K = K;
		this.Ti = Ti;
		this.Ts = Ts;
		this.Td = Td;
	}

	/*
	 * Receives an error, and a max and min value.
	 * Returns a velocity (in the bounds of max and min) that should
	 * reduce the error.
	 */
	public double getNewVelocity(double error, double min, double max)
	{
		v = v_1 + K*((error - error_1) + (Ts/Ti) * error + (Td/Ts)*(error - 2*error_1 + error_2));

		if (v < min)
			v = min;
		else if (v > max)
			v = max;

		v_1 = v;
		error_2 = error_1;
		error_1 = error;

		return v;
	}

}
