% Function that uses the PID-controller to move the joints into desired
% angles.  It also supports automatic moving of the light ball in the X
% axis.
%
% Input:    - Goal angles for each joint
%           - Number of timesteps 
%
% Output:   A matrix containing sensor values and angles of the joints for
%           each timestep in the run.

function dataRead = oneSequence(goalElbowL, goalElbowR, goalShoulderL, goalShoulderR, timeSteps);

% Make a communication object
communication = javaObject('Breve');


% Make one PID-controller for each angle that need to be controlled
pidLe = javaObject('PID', 6, 4, 4, 3);
pidRe = javaObject('PID', 6, 4, 3, 0);
pidR1 = javaObject('PID', 6, 4, 4, 1);
pidR2 = javaObject('PID', 6, 4, 4, 1);
pidR3 = javaObject('PID', 6, 4, 4, 1);
pidL1 = javaObject('PID', 6, 4, 4, 1);
pidL2 = javaObject('PID', 6, 4, 4, 1);
pidL3 = javaObject('PID', 6, 4, 4, 1);

% Set shoulder speed to zero
shoulderV = zeros(1,3);

for i = 1:timeSteps,
    
    % Get current left elbow angle
    elbowL = str2double(communication.read('localhost', 7776, 'get-left-elbow-angle'));
    % Use PID-controller to find new speed
    v = pidLe.getNewVelocity(goalElbowL-elbowL, -1, 1);
    
    % Get current right elbow angle
    elbowR = str2double(communication.read('localhost', 7776, 'get-right-elbow-angle'));
    % Use PID-controller to find new speed
    v2 = pidRe.getNewVelocity(goalElbowR-elbowR, -1, 1);
    
    % Get current right shoulder angles
    eval(communication.read('localhost', 7776, sprintf('get-right-arm-angles')));
    p = [X Y Z];
    shoulderXR = X; % Store for later use
    % Use PID-controller to find new shoulder speeds
    shoulderRV(1) = pidR1.getNewVelocity(goalShoulderR(1) - p(1), -1, 1);
    shoulderRV(2) = pidR2.getNewVelocity(goalShoulderR(2) - p(2), -1, 1);
    shoulderRV(3) = pidR3.getNewVelocity(goalShoulderR(3) - p(3), -1, 1);
    
    % Get current left shoulder angles
    eval(communication.read('localhost', 7776, sprintf('get-left-arm-angles')));
    p = [X Y Z];
    shoulderXL = X; % Store for later use
    % Use PID-controller to find new shoulder speeds
    shoulderLV(1) = pidL1.getNewVelocity(goalShoulderL(1) - p(1), -1, 1);
    shoulderLV(2) = pidL2.getNewVelocity(goalShoulderL(2) - p(2), -1, 1);
    shoulderLV(3) = pidL3.getNewVelocity(goalShoulderL(3) - p(3), -1, 1);
    
    % Send the new speeds to breve
    communication.read('localhost', 7776, sprintf('set-left-elbow-velocity_%f', v));
    communication.read('localhost', 7776, sprintf('set-right-elbow-velocity_%f', v2));
    pause(0.03);
    communication.read('localhost', 7776, sprintf('set-left-arm-velocities_%f_%f_%f', shoulderLV(1), shoulderLV(2), shoulderLV(3)));
    communication.read('localhost', 7776, sprintf('set-right-arm-velocities_%f_%f_%f', shoulderRV(1), shoulderRV(2), shoulderRV(3)));
    
    % Tell breve to iterate
    communication.read('localhost', 7776, 'matlab-iterate');
    
    % Read sensors from lightball
    eyeR = str2double(communication.read('localhost', 7776, sprintf('get-right-eye'))) / 10;
    eyeL = str2double(communication.read('localhost', 7776, sprintf('get-left-eye'))) / 10;
    
    % Store data from sensors and joint angles for current timestep
    dataRead(i,:) = [eyeL eyeR shoulderXL elbowL shoulderXR elbowR];
    pause(0.06);
    
end;