% Creates a recurrent neural network with parametric bias (RNNPB, Tani)
% with 3 layers - output, input and hidden.
% 
% Takes as input: size of the network, learning rate and momentum for the
% weights, and learning rate the PBs

function N = rnnpb(size, learning_rate, momentum, pb_learning_rate);

% size - the number of different nodes in the network
% size{1} is number of input/output
% size{2} is number of hidden nodes
% size{3} is number of context nodes
% size{4} is number of different sequences
% size{5} is number of PB nodes

% Store the values in the rnnpb object
N.size = size;
N.lr = learning_rate;
N.m = momentum;
N.lr_pb = pb_learning_rate;
N.nPB = size{5};


% Initialize weights
N.w = cell(1,2);

N.w{1} = randn(N.size{2}, N.size{1}); % Weights from hidden to output
N.w{2} = randn(N.size{1} + N.nPB + N.size{3}, N.size{2}); % Weights from input to hidden

% Initialize variables that holds the weight changes
N.dw{1} = zeros(N.size{2}, N.size{1});
N.dw{2} = zeros(N.size{1} + N.nPB + N.size{3}, N.size{2});

% Initialize contex weights
N.contextWeights =  0.7*randn(N.size{2}, N.size{3});

% Initialize PB vectors
N.PB = 0.5*ones(1, N.nPB, N.size{4});
N.aPB = N.PB;

% Initialize previous PB errors
N.ePB = zeros(1, N.nPB, N.size{4});
N.ePB_1 = zeros(1, N.nPB, N.size{4});
N.ePB_2 = zeros(1, N.nPB, N.size{4});

% Initialize change in PB
N.dPB = zeros(1, N.nPB, N.size{4});

% Create cell with three layers (i.e. the nodes)
N.L = cell(1,3);

% Create the context nodes
N.contextNodes = 0.5*ones(1, N.size{3});