% Training function for the recurrent neural network with parametric bias
% Takes as input:   net - the neural network (has to be an rnnpb.m-object)
%                   err - the error (target - output)
%                   patternNr - number (id) of the current sequence; this
%                               is used so that the correct PB vector is
%                               updated.
%
% Returns the network after one correction (one training iteration)

function net = rnnpb_train(net, err, patternNr);

% Output error
oError = net.L{3}.*(1 - net.L{3}).*err;

% Hidden error
hError = oError * net.w{1}' .* net.L{2} .* (1-net.L{2});

% The weight change
net.dw{1} = net.lr*(net.L{2}' * oError) + net.m*net.dw{1};
net.dw{2} = net.lr*(net.L{1}' * hError) + net.m*net.dw{2};

% Update weights
net.w{1} = net.w{1} + net.dw{1};
net.w{2} = net.w{2} + net.dw{2};

% Find the error in the PB vectors
pbError = zeros(net.size{5}, net.size{2});
for pbnr=1:net.size{5},
    pbError(pbnr,:) = oError * net.w{1}'.*net.w{2}(net.size{1}+pbnr,:) .* (1-net.w{2}(net.size{1}+pbnr,:));
end;

net.ePB(:,:,patternNr) = net.ePB(:,:,patternNr) - net.lr_pb .* sum((net.w{2}(net.size{1}+1:net.size{1}+net.size{5},:) .* pbError)');

% Use Simpson's rule to find the integral of the error
% (Important! These variables must be reset for each new sequence)
net.dPB = net.ePB + 4*net.ePB_1 + net.ePB_2;
net.ePB_2 = net.ePB_1;
net.ePB_1 = net.ePB;

% Update PB vectors
net.aPB = net.aPB + net.dPB;
