% Function that runs a training
% Input:    patterns1 - all the patterns (sequences)
%           nPatterns - number of different sequence patterns
%           nHidden1 - number of hidden nodes
%           nCN1 - number of context nodes
%           nPB1 - number of PB nodes
%           epochs - number of iterations to run the whole set
%           n1 - learning rate for the weights
%           a1 - momentum for the weights
%           m1 - learning rate for the PBs
%
% Returns a fully trained network

function net = run_training(patterns1, nPatterns, nHidden1, nCN1, nPB, epochs, n1, a1, m1)

sizes = cell(1,5); % Empty cell with place for 6 numbers of nodes

% nInput1 = size(patterns1); % Get number of input nodes

% Fill in the number of nodes in the size
sizes{1} = numel(patterns1(1,:,1));
sizes{2} = nHidden1;
sizes{3} = nCN1;
sizes{4} = nPatterns;
sizes{5} = nPB;

% Create the neural network
net = rnnpb(sizes, n1, a1, m1);

% Number of "chunks" in each pattern
% (A chunk is a single set of inputs in a sequence.  For example, if we
% have the sequence [0 1; 0 0; 1 1], [0 1] would be the first "chunk", ect)
nChunks1 = size(patterns1);
nChunks1 = nChunks1(1,1);


% Start training
epoch = 0;
while epoch < epochs,
    
    % Reset weight changes
    net.dw{1} = net.dw{1}.*0;
    net.dw{2} = net.dw{2}.*0;
    
    % Reset PB errors
    net.dPB = zeros(1, sizes{5}, sizes{4});
    net.ePB = zeros(1, sizes{5}, sizes{4});
    net.ePB_1 = zeros(1, sizes{5}, sizes{4});
    net.ePB_2 = zeros(1, sizes{5}, sizes{4});

    % Go through each different sequence
    for pattern=1:sizes{4},
        % Initialize contex nodes
        contexNodes1 = 0.5*ones(1, nCN1);

        % Go through each "chunk" in each sequence.
        for chunk=1:nChunks1-1,
            % Set input
            X = patterns1(chunk, :, pattern);
            
            % Activate
            net = rnnpb_activate(net, X, net.PB(:,:,pattern));
            
            % Target matrix
            T = [patterns1(chunk+1, :, pattern)];

            % Calculate error and update error sum
            net = rnnpb_train(net, T-net.L{3}, pattern);
        end;
    end;

    % Sigmoid function on the PB vector
    net.PB = 1./(1 + exp(-net.aPB));
    
    epoch = epoch + 1;
end;
